#!/usr/bin/env python

#   bibbob: a simple sound generator
#   Copyright (C) 2010  Niels Serup

#   This file is part of bibbob.
#
#   bibbob is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   bibbob is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with bibbob.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>

##''''''''''' Documentation and downloads are available at '''''''''''##
################# http://metanohi.org/projects/bibbob/ #################

#### Create waveform

# To use this program, first click on the top position on the y-axis
# (1) and then on the bottom position on the y-axis (-1).
# Then press space.
# Now mark the wave in several points.
# Press space.
# A lot of text will be printed. This can be used together with the
# synth function found in "sound.py". Also see "synthsounds.py".

import pygame
from pygame.locals import *
import sys

class WaveDrawer:
    def __init__(self, imagename):
        self.y_start = None
        self.y_end = None
        self.points = []

        pygame.display.init()

        self.image = pygame.image.load(imagename)

        self.image_width = self.image.get_width()
        self.image_height = self.image.get_height()

        self.screen = pygame.display.set_mode(
            (self.image_width, self.image_height))
        pygame.display.set_caption('Waver')

        background = pygame.Surface(self.screen.get_size()).convert()
        background.fill((255, 255, 255))
        self.screen.blit(background, (0, 0))

        self.point_marker = pygame.Surface((4, 4)).convert()
        self.point_marker.fill((0, 255, 25))

        self.image = self.image.convert()
        self.screen.blit(self.image, (0, 0))
        pygame.display.flip()

        self.clock = pygame.time.Clock()
        self.loop()

    def loop(self):
        done = False
        while not done:
            self.clock.tick(30)
            for event in pygame.event.get():
                if event.type == MOUSEBUTTONDOWN:
                    point = pygame.mouse.get_pos()
                    self.points.append(point)
                    self.screen.blit(self.point_marker,
                                     (point[0] - 2, point[1] - 2))
                    pygame.display.flip()
                elif event.type == QUIT:
                    done = True
                elif event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        done = True
                    elif event.key == K_SPACE:
                        if self.y_start is None:
                            self.y_start = self.points[0][1]
                            self.y_end = self.points[1][1]
                            self.points = self.points[2:]
                            print 'Y values have been set.'
                        else:
                            self.calculate()
                            #done = True

    def calculate(self):
        x_start = self.points[0][0]
        x_end = self.points[-1][0]
        x_width = x_end - x_start
        y_half = (self.y_end - self.y_start) / 2
        for i in range(len(self.points)):
            p = self.points[i]
            nx = float(p[0] - x_start) / x_width
            ny = -float(p[1] - self.y_start - y_half) / y_half
            self.points[i] = (nx, ny)

        nlst = []
        i = 1
        for p in self.points[:-1]:
            try:
                d = (float(self.points[i][1] - p[1]) /
                     (self.points[i][0] - p[0]))
            except ZeroDivisionError:
                d = 0.0

            nlst.extend([p[0], d, p[1]])
            i += 1
        nlst[0] = -0.001
        nnlst = []
        for i in range(0, len(nlst), 3):
            nnlst.insert(0, nlst[i + 2])
            nnlst.insert(0, nlst[i + 1])
            nnlst.insert(0, nlst[i])

        print self.points
        print nnlst

if __name__ == '__main__':
    try:
        wd = WaveDrawer(sys.argv[1])
    except:
        sys.stderr.write("You haven't specified a picture!")
