#!/usr/bin/env python
# Global
from PIL import Image, ImageDraw
import colorsys
import numpy
import math
# Local
from music import *
from sound import *

POW15 = 2**15
POW15M1 = POW15 - 1
POW16 = POW15 * 2

class ImageSound:
    def __init__(self, infile, outfile, play):
        self.mus = Music(channels=1)
        self.infilename = infile
        self.outfilename = outfile
        self.do_play = play

    def start(self):
        self.infile = Image.open(self.infilename)
        self.data = numpy.asarray(self.infile.getdata())
        self.run_analysis()
        self.mus.save(self.outfilename)

    def run_analysis(self):
        for i in xrange(len(self.data)):
            color = self.rgb2hsl(self.data[i])
            h = color[0]
            s = color[1]
            l = color[2]

            self.add_wave(200 / s, l * 1000, (h - 0.5))

    def rgb2hsl(self, rgba):
        rgb = list(rgba)[:3]
        return colorsys.rgb_to_hls(*[x/255.0 for x in rgb])

    def add_wave(self, frequency, amplitude, distortion):
        if math.isinf(frequency): return
        length = 1000.0 / frequency
        m = self.mus.create(SineWave, length, amplitude, frequency)
        for i in xrange(m.size):
            temp = m.data[i] + int(distortion * POW16)
            if temp > POW15M1:
                m.data[i] = POW15M1
            elif temp < -POW15:
                m.data[i] = -POW15
            else:
                m.data[i] = temp
        self.mus.add(m)


if __name__ == '__main__':
    import sys
    from optparse import OptionParser
    parser = OptionParser(prog='image2sound',
                          usage='Usage: %prog [OPTION]... IMAGEFILE',
                          description='Converts an image to sound', version="""\
%prog 0.1
Copyright (C) 2010  Niels Serup
License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.""", epilog='\
If no OUTFILE is specified, the sound will be played.')
    parser.add_option('-o', dest='outfile', metavar='OUTFILE',
                      help='specify output file')
    parser.add_option('-p', dest='play', action='store_true',
                      help='play the sound even if it is also saved to a file')
    (options, args) = parser.parse_args()
    try:
        filename = args[0]
    except Exception:
        parser.error('no IMAGEFILE specified')

    if not options.outfile:
        options.play = True
    imgsnd = ImageSound(filename, options.outfile, options.play)
    try:
        imgsnd.start()
    except (KeyboardInterrupt, EOFError):
        print
    finally:
        pass#imgsnd.end()
