#!/usr/bin/env python

#   bibbob: a simple sound generator
#   Copyright (C) 2010  Niels Serup

#   This file is part of bibbob.
#
#   bibbob is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   bibbob is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with bibbob.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>

##''''''''''' Documentation and downloads are available at '''''''''''##
################# http://metanohi.org/projects/bibbob/ #################

#### Music

import numpy
import pyaudio

class Music:
    def __init__(self, **settings):
        def get(name, default):
            try: return settings[name]
            except Exception: return default
        self.channels = get('channels', 2)
        self.rate = get('rate', 44100)

        self.sounds = numpy.empty(0, dtype=numpy.int16)
        self.length = 0

    def create(self, Class, length, *args):
        sound = Class(self.channels, self.rate, length)
        sound.create(*args)
        return sound

    def add(self, sound):
        self.sounds = numpy.append(self.sounds, sound.data)
        self.length += sound.length

    def put(self, music):
        self.sounds = numpy.append(self.sounds, music.sounds)
        self.length += music.length

    def combine(self, *music):
        if len(music) > 1:
            for x in music:
                self.combine(x)
            return
        if len(music) == 0:
            return
        music = music[0]
        l1 = len(self.sounds)
        l2 = len(music.sounds)

        if l1 > l2:
            l = l2
            nl = l1
            m = self.sounds
        else:
            l = l1
            nl = l2
            m = music.sounds

        s1 = self.sounds
        s2 = music.sounds
        for i in xrange(l):
            m[i] = (s1[i] + s2[i]) / 2

        self.sounds = m
        self.length = nl

    def play(self):
        samps = self.sounds.tostring()
        pa = pyaudio.PyAudio()
        strm = pa.open(
            format = pyaudio.paInt16,
            channels = self.channels,
            rate = self.rate,
            output = True)
        strm.write(samps)
        strm.close()
        pa.terminate()

    def get_simplified_waveform_coordinates(self, width=800, height=600):
        d = float(width) / height
        all_sounds = list(self.sounds)
        chans = []
        for x in range(self.channels):
            sounds = []
            for y in xrange(0, len(all_sounds), self.channels):
                sounds.append(all_sounds[y + x])
            maxval = sounds[0]
            minval = sounds[0]
            for y in sounds:
                if y > maxval: maxval = y
                if y < minval: minval = y
            theight = maxval - minval
            twidth = len(sounds)

            wtimes = twidth / width
            if wtimes == 0: wtimes = 1
            hfac = float(height) / theight

            i = 0
            coors = []
            for y in range(0, len(sounds), wtimes):
                coor_y = int((sounds[y] - minval) * hfac)
                i += 1
                coors.append(coor_y)
            coors = coors[:width]
            chans.append(coors)

        return chans
