#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   cheatbibtex: a script that aids in a better use of BiBTeX in LaTeX
#   Copyright (C) 2010  Niels Serup

#   This file is part of cheatbibtex.
#
#   cheatbibtex is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   cheatbibtex is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with cheatbibtex.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>

##''''''''''' Documentation and downloads are available at '''''''''''##
############### http://metanohi.org/projects/cheatbibtex/ ##############

                                 ############
                                 ## Parser ##
                                 ############

#############
## IMPORTS ##
#############
## Global
import re

## Local
from cheatbibtexlib.err import *

######################
## CLASS DEFINITION ##
######################
class Parser:
    """Parses data"""
    def __init__(self, latexdata, styledata):
        self.lat = latexdata
        self.sty = styledata

        self.bibdata = {}
        self.stydata = {}

        self.parse_style()
        self.parse_latex()
        self.parse_bibtex()

    def parse_style(self):
        sty = self.sty.split('\n\n')
        sty = [x.strip() for x in sty]
        for x in sty:
            xf = x.find('\n')
            name = x[1:xf]
            content = x[xf+1:]
            if name == '#details':
                self.stydata[name] = eval(content)
            else:
                self.stydata[name] = content

    def remove_latex_comments(self):
        self.lat = re.sub(r'%.*', '', self.lat)

    def parse_latex(self):
        self.remove_latex_comments()
        inputs = re.findall(r'\input{.+}', self.lat)
        inputs = [open(x[6:-1] + '.tex', 'U') for x in inputs]
        for x in inputs:
            self.lat += x.read()
        [x.close() for x in inputs]
        self.remove_latex_comments()

        cites = re.findall(r'\\[^{]*cite[^}]*{.+?}', self.lat)
        for i in range(len(cites)):
            x = cites[i]
            x = x[x.find('{')+1:]
            x = x[:x.find('}')]
            if x.startswith('#'):
                cites[i] = None
            else:
                cites[i] = x
        while 1:
            try:
                cites.remove(None)
            except:
                break

        self.used_sources = list(set(cites))

        bibs = re.findall(r'\printbib{.+}', self.lat)
        bibs = [x[9:-1] for x in bibs]
        self.rawbibdata = {}
        for x in bibs:
            f = open(x + '.bib', 'U')
            self.rawbibdata[x] = f.read()
            f.close()

    def parse_bibtex(self):
        self.bibdata['*all*'] = {}
        for x in self.rawbibdata.items():
            i = self.parse_bibtex_data(x[1], x[0])
            self.bibdata[x[0]] = i
            for x in i.items():
                self.bibdata['*all*'][x[0]] = x[1]

    def parse_bibtex_data(self, data, name):
        data = data.split('\n@')
        ndata = {}
        for x in data:
            inf = self.parse_bibtex_entry(x, name)
            if inf:
                ndata[inf[0]] = inf[1]
        return ndata

    def parse_bibtex_entry(self, data, filename):
        ff = data.find('{')
        typ = data[:ff].lower().strip()
        if typ.startswith('@'): typ = typ[1:]
        fc = data.find(',')
        nam = data[ff+1:fc]
        if nam not in self.used_sources:
            return None

        data = data[fc+2:]
        data = data.split(',\n')
        ndata = {'type': typ, 'file': filename}

        for x in data:
            x = x.split('=', 1)
            if len(x) == 2:
                ndata[x[0].strip().lower()] = self.parse_bibtex_value(x[1])

        if 'author' in ndata:
            ndata['authors'] = ndata['author'].split(' and ')
            backnames = []
            for x in ndata['authors']:
                fs = x.find(' ')
                bu1 = x.find('{')
                bu2 = x.find('}')
                if bu1 != -1 and bu2 != -1:
                    fs = x.find(' ', bu2)
                if fs == -1:
                    bc = x
                else:
                    bc = x[fs+1:] + ', ' + x[:fs]
                backnames.append(bc)
            ndata['backnames'] = backnames[0]
            l = len(backnames)
            for i in range(1, l):
                if l - i > 1:
                    ndata['backnames'] += self.stydata['#details'][0] + backnames[i]
                else:
                    ndata['backnames'] += self.stydata['#details'][1] + backnames[i]
        return (nam, ndata)

    def parse_bibtex_value(self, val):
        val = val.replace('[', '').replace(']', '').strip()
        if val[0] == '"':
            val = val[1:]
        if val.endswith('"\n}'):
            val = val[:-3].rstrip()
        if val[-1] == '"':
            val = val[:-1]
        val = re.sub(r'\s+', ' ', val)
        return val
