#!/usr/bin/python
# data2png: saves byte data to a png file as a square
# Public domain if legally possible

import sys, math, png
args = sys.argv[1:]
if not args:
    sys.stderr.write('No input file specified.\n')
    sys.exit()

if '-h' in args or '--help' in args:
    print """

    data2png reads data from a file and saves it as a PNG image in a square
    form. A file converted to png takes up about the same disk space as the
    same file compressed with gunzip would.

    Usage: data2png [OPTION] INFILE [OUTFILE]

    Options:
        -u, --unpack   convert PNG back to data
        -s, --simple   do not use an alpha channel
        -h, --help     Display this help

    If '-' is specified as INFILE, data2png will attempt to read from standard
    in. If '-' is specified as OUTFILE, data2png will attempt to write to
    standard out. Typing '\-' will escape a dash.

    If no OUTFILE is specified, "INFILE.png" will be used when packing, while
    "data" will be used when unpacking. If the input comes from standard in,
    "data.png" will be used when packing.

"""
    sys.exit()

PACK = True
ALPHA = True

if args[0] == '-u' or args[0] == '--unpack':
    PACK = False
    args = args[1:]
elif args[0] == '-us' or args[0] == '-su':
    PACK = False
    ALPHA = False
    args = args[1:]

if args[0] == '-s' or args[0] == '--simple':
    ALPHA = False
    args = args[1:]
elif args[0] == '-us' or args[0] == '-su':
    PACK = False
    ALPHA = False
    args = args[1:]

if not args:
    sys.stderr.write('No input file specified.\n')
    sys.exit()

infile = args[0]
if infile == '-' and not sys.stdin.isatty():
    ifl = sys.stdin
    infile = 'data'
else:
    if infile == '\\-u' or infile == '\\--unpack' or infile == '\\-s' or infile == '\\--simple':
        infile = infile[1:]
    elif infile == '\\-':
        infile = '-'
    try:
        ifl = open(infile, 'rb')
    except:
        sys.stderr.write('Input file doesn\'t work.\n')
        sys.exit()

args = args[1:]
if args:
    outfile = args[0]
elif PACK:
    outfile = infile + '.png'
else:
    outfile = 'data'

if outfile == '-':
    ofl = sys.stdout
else:
    try:
        ofl = open(outfile, 'wb')
    except:
        sys.stderr.write('Output file doesn\'t work.\n')
        sys.exit()


if PACK:
    if ALPHA:
        c = 4
    else:
        c = 3
    b = ifl.read() + chr(128) + chr(127); ifl.close()
    bl = len(b)
    rl = int(math.ceil(math.sqrt(float(bl) / c)))
    brl = rl * c
    d = int(math.pow(rl, 2) * c - bl)

    #b += chr(0) * d
    e = 0
    hbl = int(math.ceil((bl) / 2))
    for i in range(d):
        while 1:
            t = b[hbl+e+i % bl]
            if ord(t) == 128:
                e += 1
            else:
                break
        b += t

    bl += d

    p = []
    for x in range(0, bl, brl):
        p.append([ord(y) for y in b[x:x+brl]])

    w = png.Writer(width=rl, height=rl, alpha=ALPHA)
    w.write(ofl, p)
    ofl.close()

else:
    s = png.Reader(file=ifl)
    if ALPHA:
        s = s.asRGBA()
    else:
        s = s.read()
    b = list(s[2])
    #l = s[3]['size'][0] * s[3]['size'][1]
    nb = []
    for x in b:
        nb.extend(x)
    b = nb
    del nb
    
    bl = len(b)
    while bl > -1:
        bl -= 1
        if b[bl] == 128:
            break
    b = b[:bl]
    b = ''.join([chr(y) for y in b])
    ofl.write(b)
    ofl.close()

