#!/usr/bin/env python

# This program is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What The Fuck You Want
# To Public License, Version 2, as published by Sam Hocevar. See
# http://sam.zoy.org/wtfpl/COPYING for more details.

import sys
import math
import wave
import struct

args = sys.argv[1:]
if not args:
    sys.stderr.write('No input file specified.\n')
    sys.exit()

if '-h' in args or '--help' in args:
    print """\
data2wav reads data from a file and saves it as a WAV sound file.

Usage: data2wav [OPTION] INFILE [OUTFILE]
Options:
    -u, --unpack   convert wav back to data
    -h, --help     Display this help

If '-' is specified as INFILE, data2wav will attempt to read from standard
in. If '-' is specified as OUTFILE, data2wav will attempt to write to
standard out. Typing '\-' will escape a dash.

If no OUTFILE is specified, "INFILE.wav" will be used when packing, while
"data" will be used when unpacking. If the input comes from standard in,
"data.wav" will be used when packing.
"""
    sys.exit()

PACK = True

if args[0] == '-u' or args[0] == '--unpack':
    PACK = False
    args = args[1:]

infile = args[0]
if infile == '-' and not sys.stdin.isatty():
    if not PACK:
        sys.stderr.write("You have to load a wave file.\n")
        sys.exit()
    else:
        ifl = sys.stdin
        infile = 'data'
else:
    if infile == '\\-u' or infile == '\\--unpack':
        infile = infile[1:]
    elif infile == '\\-':
        infile = '-'
    if not PACK:
        try:
            ifl = wave.open(infile, 'r');
        except:
            sys.stderr.write('Input file doesn\'t work.\n')
            sys.exit()
    else:
        try:
            ifl = open(infile, 'rb')
        except:
            sys.stderr.write('Input file doesn\'t work.\n')
            sys.exit()

args = args[1:]
if args:
    outfile = args[0]
elif PACK:
    outfile = infile + '.wav'
else:
    outfile = 'data'

if outfile == '-':
    if PACK:
        sys.stderr.write("You have to save this to a wave file.\n")
        sys.exit()
    else:
        ofl = sys.stdout
else:
    if PACK:
        try:
            ofl = wave.open(outfile, 'w')
        except:
            sys.stderr.write('Output file doesn\'t work.\n')
            sys.exit()
    else:
        try:
            ofl = open(outfile, 'wb')
        except:
            sys.stderr.write('Output file doesn\'t work.\n')
            sys.exit()


if PACK:
    frek = 440;
    frate = 11025.0

    data = ifl.read()
    ifl.close()

    nchannels = 1
    sampwidth = 2
    framerate = int(frate)
    nframes = len(data)
    comptype = "NONE"
    compname = "not compressed"

    ofl.setparams((nchannels, sampwidth, framerate, nframes, comptype, compname))

    for x in data:
        ofl.writeframes(struct.pack('H', ord(x)*256))
    ofl.close()
else:
    nf = ifl.getnframes()
    data = ifl.readframes(ifl.getnframes())
    for i in range(0, len(data)-1, 2):
        x = data[i:i+2]
        ofl.write(chr(struct.unpack('H', x)[0]/256))
    ofl.close()
