#!/usr/bin/env python

#   gadon: gadon, gadon, gadon
#   Copyright (C) 2010  Niels Serup

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>
#   v0.0.0.1.2.3 (initial release)
##################################################

# Imports
import sys
import random
import os.path
import pygame
from pygame.locals import *

# "Constants"
VERSION = '0.0.0.1.2.3'

class Circle:
    def __init__(self, world):
        self.world = world
        self.max_radius = random.randint(self.world.max_size / 10,
                                         self.world.max_size / 3)
        self.min_radius = random.randint(self.max_radius / 6,
                                         self.max_radius / 4)
        mr = self.max_radius
        self.pos = [random.randint(mr, self.world.size[0] - mr),
                    random.randint(mr, self.world.size[1] - mr)]

        self.up = random.randint(0, 1)
        radius_diff = self.max_radius - self.min_radius
        self.speed = [-radius_diff / 80, radius_diff / 8]
        if self.up:
            self.radius = self.min_radius
        else:
            self.radius = self.max_radius

        self.color_max = [360, 100, 100, 100]
        self.color_chg = [3.6, 1, 1, 1]
        self.color = [random.randint(0, 360), random.randint(0, 100),
                      random.randint(0, 100), random.randint(0, 100)]

    def update(self):
        self.radius += self.speed[self.up]
        if self.radius > self.max_radius:
            self.radius = self.max_radius
            self.up = 0
        elif self.radius < self.min_radius:
            self.radius = self.min_radius
            self.up = 1

        for i in range(4):
            self.color[i] = (self.color[i] +
            self.color_chg[i]) % self.color_max[i]

    def draw(self):
        self.world.draw_circle(self.pos, self.radius, self.color)

# Art object
class GraphicsBoard(object):
    def __init__(self, options):
        for x in eval(str(options)).items():
            self.__setattr__(*x)
        self.size = (self.width, self.height)
        self.max_size = min(self.size)
        self.center = (self.width / 2, self.height / 2)

    def start(self):
        pygame.display.init()

        # Create board
        if self.show:
            self.screen = pygame.display.set_mode(self.size)
            pygame.display.set_caption('gadon %s' % VERSION)
        else:
            pygame.display.set_mode((1, 1))
            self.screen = pygame.Surface(self.size)

        # Create background
        self.bgsurf = pygame.Surface(
            [s * self.antialias for s in self.size]).convert_alpha()
        if self.antialias > 1:
            self.vis_bgsurf = \
                pygame.Surface(self.size).convert_alpha()


        if self.outdir and not os.path.exists(self.outdir):
            try:
                os.path.mkdir(self.outdir)
            except Exception:
                sys.stderr.write('could not create directory "%s"\n' %
                                 self.outdir)
                sys.exit(1)
        # Wait until user quits
        self.clock = pygame.time.Clock()
        self.run()

    def draw_circle(self, pos, radius, color):
        ncolor = pygame.Color(0)
        ncolor.hsla = color
        pygame.draw.circle(self.bgsurf, ncolor,
                           [p * self.antialias for p in pos],
                           radius * self.antialias, 0)

    def run(self):
        done = False
        circs = []
        for i in range(self.circles_n):
            circs.append(Circle(self))

        runs = 0
        while not done:
            if 0 < self.max_runs <= runs:
                break
            self.clock.tick(30)
            self.bgsurf.fill(self.background)
            for x in circs:
                x.draw()
            if self.antialias > 1:
                pygame.transform.smoothscale(self.bgsurf,
                                             self.size, self.vis_bgsurf)
                self.screen.blit(self.vis_bgsurf, (0, 0))
            else:
                self.screen.blit(self.bgsurf, (0, 0))
            if self.show:
                pygame.display.flip()
            if self.outdir:
                try:
                    pygame.image.save(self.screen, os.path.join(
                            self.outdir, '%08d.%s' %
                            (runs, self.filetype)))
                except Exception, e:
                    sys.stderr.write('python error: %s\n' % str(e))
                    pass
            for x in circs:
                x.update()
            c = circs.pop()
            circs.insert(0, c)

            runs += 1
            for event in pygame.event.get():
                if event.type == QUIT or \
                   (event.type == KEYDOWN and \
                     (event.key == K_ESCAPE or event.key == K_q)):
                    done = True

    def end(self):
        pass



#############################################
### Starting metart from the command line ###
if __name__ == '__main__':
    from optparse import OptionParser
    parser = OptionParser(prog='gadon',
                          usage='Usage: %prog [OPTION]...',
                          description='Gadon, gadon, gadon', version="""\
%prog """ + VERSION + """
Copyright (C) 2010  Niels Serup
License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.""", epilog='\
If no OUTDIR is specified, graphics will be shown directly.')
    parser.add_option('-o', '--outdir', dest='outdir', metavar='OUTDIR',
                      help='specify output directory')
    parser.add_option('-t', '--filetype', dest='filetype', metavar='TGA|PNG|JPG|BMP',
                      help='specify output filetype', default='png')
    parser.add_option('-s', '--show', dest='show', action='store_true',
                      help='show the animation even if it is saved to \
a file')
    parser.add_option('-m', '--max-runs', dest='max_runs', metavar='NUMBER',
                      help='specify max runs (default is -1, meaning \
it will never stop)', default=-1, type='int')
    parser.add_option('-x', '--width', dest='width', type='int',
                      metavar='NUMBER', default=800,
                      help='set width of graphics board')
    parser.add_option('-y', '--height', dest='height', type='int',
                      metavar='NUMBER', default=600,
                      help='set height of graphics board')
    parser.add_option('-n', '--circles', dest='circles_n',
                      metavar='NUMBER', default=10, type='int',
                      help='set number of circles to be displayed')
    parser.add_option('-b', '--background', dest='background',
                      metavar='R,G,B', default='0,0,0',
                      help='set background color of graphics board')
    parser.add_option('-a', '--antialias', dest='antialias', type='int',
                      metavar='NUMBER', default=2,
                      help='set antialiasing number. The higher, the \
nicer. Must not be lower than 1. Setting it to 1 disables antialiasing')

    (options, args) = parser.parse_args()

    if options.filetype:
        options.filetype = options.filetype.lower()
    if not options.outdir:
        options.show = True

    options.background = [int(x) for x in tuple(options.background.split(','))]

    art = GraphicsBoard(options)
    try:
        art.start()
    except (KeyboardInterrupt, EOFError):
        print
    finally:
        art.end()
