#!/usr/bin/python

# problist: A combinatorial, probalistic list generator

import sys

# What ASCII values to be included
a_from = 32
a_to = 126

# What string lengths to be included
l_from = 1
l_to = 3

system_call = ''
no_print = False

if len(sys.argv) > 1:
    info = sys.argv[1]
    if info == '--help':
        print \
"""Usage: problist [OPTIONS]
problist is a combinatorial, probalistic list generator. It can be used to
generate lists in this form: a, b, c, aa, ab, ac, ba, bb, bc, ca, cb, cc.

Options:
  --help      Display this help and exit.
  all others  problist accepts options in the form of
              option1[value1],option2[value2],option3[value3]...

Options are two characters long and do not always require values.
  af<FROM>     Set lower ASCII value limit to <FROM>. Default is 32 (space).
  at<TO>       Set upper ASCII value limit to <TO>. Default is 126 (tilde).
  lf<FROM>     Set lower string length limit to <FROM>. Default is 1.
  lt<TO>       Set upper string length limit to <TO>. Default is 3.
  sc<PROGRAM>  Make system calls to <PROGRAM> with strings as first arguments.
  np           Do not print strings.

Examples:
    $ problist lt100
      This will not end in your lifetime.
    $ problist af72,at73,np,sctoilet
      This will send H, I, HH, HI, IH, II, HHH, HHI, HIH,
      HII, IHH, IHI, IIH and III to the toilet program."""
        sys.exit()
    else:
        spl = info.replace('\\,', '\\`').split(',')
        for x in spl:
            x = x.replace('\\`', ',')
            s = x[0:2]
            e = x[2:]
            try: i = int(e)
            except: i = 0
            if s == 'af':
                a_from = i
            elif s == 'at':
                a_to = i
            elif s == 'lf':
                l_from = i
            elif s == 'lt':
                l_to = i
            elif s == 'sc':
                system_call = e
            elif s == 'np':
                no_print = True

# Make values work with Python's range() function
a_to += 1
l_to += 1

if not system_call:
    if not no_print:
        def process(txt):
            print txt
    else:
        def process(txt):
            pass
else:
    import os
    
    def ready(txt):
        r = (
        '\\',
        '"',
        "'",
        ' ',
        '(',
        ')',
        ';',
        '*',
        '?',
        '<',
        '>',
        '`',
        '|',
        '$',
        '!',
        '#',
        '&',
        '~'
        )
        for x in r:
            txt = txt.replace(x, '\\'+x)
        return txt
    
    system_call += ' '
    if not no_print:
        def process(txt):
            global system_call
            print txt
            os.system(system_call+ready(txt))
    else:
        def process(txt):
            global system_call
            os.system(system_call+ready(txt))


# This is the function that does the tough work
def pc(l, c):
    global a_from, a_to
    cl = len(c)
    for i in range(a_from, a_to):
        if l != cl:
            pc(l, c + chr(i))
        else:
            process(c)
            break

# Start pc
for l in range(l_from, l_to):
    pc(l, '')
