#!/usr/bin/python
# search.py: a program that searches through files
# Copyright (C) 2009  Niels Serup

# This program is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What The Fuck You Want
# To Public License, Version 2, as published by Sam Hocevar. See
# http://sam.zoy.org/wtfpl/COPYING for more details.

# You can contact me at ns@metanohi.org

# Version 0.1

# This program is meant for searching in local webpages and will attempt to
# fetch the title of a page if that page ends up in the search results.

import os
import sys
import time
import platform
import threading

###################
# FUNCTIONS BEGIN #
# The following code consists purely of functions. Some of the functions rely
# on global variables that are specified in the bottom of this program.

def inp2arr(txt):
  # This function takes a string and converts it to a list consisting of
  # strings. The function follows basic terminal-like escaping, allowing both
  # "foo bar" and foo\ bar as one "word", while foo bar will be two "words".
  
  inp = ' ' + txt.strip() + ' '
  inps = list()
  inp_len = len(inp)
  
  quot = '' # No quotes at start    
  p = 0 # Starting point is 0

  # Looping through characters.. characters.. characters..
  for x in range(1, inp_len):
    curr = inp[x] # Current character
    prev = inp[x-1] # Previous character

    # To quote or not to quote
    if quot == '' and prev == ' ' and (curr == '"' or curr == "'"):
      quot = curr
    elif (curr == '"' and quot == '"') or (curr == "'" and quot == "'"):
      quot = ''

    # Various checking
    if curr == ' ' and quot == '' and prev != '\\' and prev != ' ':
      toadd = inp[p:x].strip()
      if (toadd[0] == '"' and toadd[-1:] == '"') or (toadd[0] == "'" and toadd[-1:] == "'"):
        toadd = toadd[1:-1]
      toadd = toadd.replace('\\ ', ' ')
      inps.append(toadd)
      p = x # The former ending point turns into the new starting point

  if p < x: # If inp looks like "do 'nil" (odd-placed comment before letters)
    inps.append(inp[p:x].strip())
  return inps

def loop(arg, dir, files):
  # This function loops through files in dir
  
  for f in files:
    fn = dir + '/' + f # Complete file name
    if os.path.isfile(fn):
      of = open(fn, 'r')
      fr = of.read()
      
      #num = find(strip_tags(fr).lower()) # VERY slow
      num = find(fr.lower()) # FAIRLY FAST
      #num = find(fr) # FASTEST
      if num > 0:
        places.append([num, fn]) # Add to list if match

def find(str):
  # This function searches through a string for inputs specified by the user
  # and returns a magical number that identifies how well a match that string
  # was compared to the user input.
  
  fnum = 1 # "Final number" -- higher fnum = better match
  
  for cinp in q:
    # Loop through input as split by inp2arr()
    num = 0
    tmp = str.find(cinp) # Position of input
    while tmp != -1: # Keep on going until there's no match
      start = tmp + 1
      tmp = str.find(cinp, start)
      num += 1 # One more match = 1 added to num
    fnum *= num # Increase final number
  
  return fnum

def strip_tags(html):
  # This function attempts to remove all XML type tags.
  # The speed efficiency of this function should be fairly good.
  
  # As a default, this function is not used by search.py, as it is too slow.
  
  strpos = 0 # The string starts at 0
  while strpos != -1: # Loop while there're still tags
    start = html.find('<', strpos) # Start by finding a tag start
    end = html.find('>', start) + 1 # Find the tag end
    html = html[:start] + html[end:] # Remove the tag from the html var
    strpos = html.find('<', end - start) # Next tag
  return html

def gettitle(f):
  # This function attempts to retrive the title of a (X)HTML document in a
  # simple and fairly efficient way.
  
  str = open(f, 'r').read()
  ftit = str.find('<title>') # Start
  if ftit != -1:
    ftitb = str.find('</title>', ftit) # End
    if ftitb != -1:
      title = str[ftit + 7:ftitb] # Between start and end
    else:
      title = f # Fallback
  else:
    title = f # Fallback
  
  return title

class thread(threading.Thread):
    def __init__(self, what):
        self.what = what
        threading.Thread.__init__(self)
        
    def run(self):
        os.system('icecat ' + self.what)

# FUNCTIONS END #
#################


    # --- #


##############
# MAIN BEGIN #
# This is the part of the program that is executed on launch.
# Several global variables are declared here.

max_results = 16 # The max number of search results to display
params = sys.argv[1:] # If input
startover = False # Quit on quit

# Infinite loop coming up
while 1:
  if params and not startover:
    q = params
  else:
    q = inp2arr(raw_input('Enter search string: '))
  
  for i in range(len(q)):
    # These replaces are optimized for the Danish language with its odd letters
    q[i] = q[i].lower().replace('ae', '\xc3\xa6').replace('\x91', '\xc3\xa6').replace('\x92', '\xc3\xa6').replace('oe', '\xc3\xb8').replace('\x9B', '\xc3\xb8').replace('\x9D', '\xc3\xb8').replace('aa', '\xc3\xa5').replace('\x86', '\xc3\xa5').replace('\x8F', '\xc3\xa5')
  
  startover = False

  start = time.time() # = NOW
  print 'Searching...'
  places = [] # Start empty
  os.path.walk('.', loop, None) # Add to places
  places.sort()
  places.reverse() # Highest value on top of list

  print 'The search took ', time.time() - start, 'seconds.'
  stop = False
  if not places:
    print 'No results'
    ri = raw_input('Press Enter to search anew or enter "q" to quit: ')
    if ri == 'q':
      break
    else:
      continue
  
  if max_results > len(places):
    results_len = len(places)
  else:
    results_len = max_results
  while 1:
    print 'This is a list of the top ' + str(results_len) + ' results:'
    i = 0
    for x in places:
      if i == max_results:
        break
      # Danish language
      print ("0" + str(i))[-2:], gettitle(x[1]).replace('\xc3\xa6', '\x91').replace('\xc3\x86', '\x91').replace('\xc3\xb8', '\x9B').replace('\xc3\x98', '\x9B').replace('\xc3\xa5', '\x86').replace('\xc3\x85', '\x86')
      i += 1
    
    # The rest is self explanatory (= I'm too lazy to actually comment it)
    num = -1
    while num == -1:
      try:
        inp = raw_input('\nEnter a number between 0 and ' + str(results_len - 1) + ' (or enter "q" to quit, or enter "n" to do a new search): ')
        if inp == 'q':
          stop = True
          break
        elif inp == 'n':
          stop = True
          startover = True
          break
        num = int(inp)
        if num < 0 or num > results_len - 1:
          num = -1
          print 'Enter a number below ' + str(results_len) + ' but above -1.'
      except:
        num = -1
        print 'Enter a number!'
    
    if stop:
      break
    nf = places[num][1]
    if platform.system() == 'Windows':
      nf = nf.replace('/', '\\') # M$ Windows has odd habits
      
    print 'Opening file', nf
    try: os.startfile(nf)
    except:
      tmpsys = thread(nf)
      tmpsys.start()
    print '\n'
    
  if not startover:
    break # Quit/exit/die/leave/kill

# MAIN END #
############
