#!/usr/bin/env python

#   wikiaq: a quiz system fetching facts from Wikipedia
#   Copyright (C) 2010  Niels Serup
#
#   This file is part of wikiaq.
#
#   wikiaq is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   wikiaq is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with wikiaq.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>
#   See http://metanohi.org/projects/wikiaq/


import sys
import readline
from optparse import OptionParser
from wikiaqlib.dataquizzer import *

def smart_split(text, length):
    n = []
    while 1:
        l = length
        if l >= len(text):
            n.append(text)
            break

        while l > 0 and text[l] != ' ':
            l -= 1
        n.append(text[:l])
        text = text[l + 1:]
    return n

parser = OptionParser(usage='Usage: %prog [FILE]',
                      version="""\
wikiaq 0.1
Copyright (C) 2010  Niels Serup <ns@metanohi.org>
License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.""")
(options, args) = parser.parse_args()
if len(args) == 0:
    args.append('wikiqas')
filename = args[0]

dq = DataQuizzer(filename)

ll = 4
indent_space = ll * ' '
exit = False
try:
    while 1:
        q = dq.new_question()
        pick = q.get_pick()
        print pick[2] + '...'
        cs = q.get_choices()
        i = 0
        for x in cs:
            text = x[4]
            if not text.endswith('.'):
                text += '.'
            text += ' '
            ts = text.find('.')
            while ts != -1 and text[:ts].endswith('cf') or text[:ts].endswith('St'):
                ts = text.find('.', ts + 1)
            if ts != -1:
                ts += 1
            text = x[3] + ' ' + text[:ts]
            linestart = str(i) + '.'
            text = smart_split(text, 79 - ll)
            ntext = linestart
            for x in text:
                ntext += '\n' + indent_space + x
            print ntext
            i += 1
        while 1:
            inp = raw_input('>>> ')
            if inp == 'exit':
                exit = True
                break
            try:
                inp = int(inp)
                if inp > -1 and inp < i:
                    break
                print 'Number not in range. Try again.'
            except Exception:
                print 'Not a number. Try again.'
        if exit:
            break

        ans = q.answer(inp)
        if ans:
            print 'Congratulations! Your answer was correct!',
        else:
            print 'Your answer was incorrect.',
        text = pick[2] + ' ' + pick[3] + ' ' + pick[4] + '.'
        text = smart_split(text, 79 - ll)
        ntext = ''
        for x in text:
            ntext += '\n' + indent_space + x
        ntext += '\n' + indent_space + '  <' + pick[0] + '>'
        print ntext

        print '#' * 79
        i = 0
        for x in cs:
            print str(i) + '.', x[1], '<' + x[0] + '>'
            i += 1
        print '#' * 79

        raw_input()
        print '*' * 79
        print

except KeyboardInterrupt:
    print
