#!/usr/bin/env python

#   wikiaq: a quiz system fetching facts from Wikipedia
#   Copyright (C) 2010  Niels Serup
#
#   This file is part of wikiaq.
#
#   wikiaq is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   wikiaq is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with wikiaq.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>
#   See http://metanohi.org/projects/wikiaq/


import sys
import re
import time
from fakeopener import *

class DataGetter:
    def __init__(self, url, afile):
        self.url = url
        self.tags_re = re.compile(r'<.+?>', re.DOTALL)
        self.refs_re = re.compile(r'\[\d+\]', re.DOTALL)
        self.content_re = re.compile(r'(.+?) (is|was|are|were|refers to) (.+)')
        self.file = afile

    def start(self):
        while 1:
            self.get_content()
            time.sleep(1)

    def get_content(self):
        place = fakeurlopen(self.url)
        url = place.geturl()
        data = place.read()

        a = data.find('<table class="infobox vcard"')
        if a != -1:
            data = data[data.find('</table>', a) + 10:]

        data = data[data.find('<p>', data.find('<!-- start content -->')) + 3:]
        data = data[:data.find('</p>')]

        focus = data.find('<b>')
        if focus != -1:
            focus += 3
            focus2 = data.find('</b>', focus)
            if focus2 != -1:
                focus = data[focus:focus2]
                focus = self.tags_re.sub('', focus)
                focus = self.refs_re.sub('', focus)
            else: focus = ''
        else: focus = ''

        data = self.tags_re.sub('', data)
        data = self.refs_re.sub('', data)

        rem = self.content_re.match(data)
        try:
            data = [url, focus]
            data.extend(rem.groups())
            data = str(tuple(data))
            print data
            self.file.write(data + ',\n')
        except Exception:
            pass

