#!/usr/bin/env python

#   wikiaq: a quiz system fetching facts from Wikipedia
#   Copyright (C) 2010  Niels Serup
#
#   This file is part of wikiaq.
#
#   wikiaq is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   wikiaq is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with wikiaq.  If not, see <http://www.gnu.org/licenses/>.

#   Maintained by Niels Serup <ns@metanohi.org>
#   See http://metanohi.org/projects/wikiaq/


import sys
import random
import re

def _callback(matches):
    id = matches.group(1)
    try:
        return unichr(int(id))
    except:
        return id

def decode_unicode_references(data):
    return re.sub('&#(\d+);', _callback, data)

def unescape(s):
    s = s.replace('&lt;' '<')
    s = s.replace('&gt;', '>')
    s = s.replace('&amp;', '&')
    return s

class OutWithDecoding:
    def __init__(self):
        sys.stdout = self

    def write(self, msg):
        try:
            sys.__stdout__.write(decode_unicode_references(unescape(msg)).encode('utf8'))
        except Exception:
            sys.__stdout__.write(msg)

class Question:
    def __init__(self, data, choices_num = 5):
        lenm1 = len(data) - 1
        self.pick_num = random.randint(0, lenm1)
        self.pick = data[self.pick_num]

        nums_taken = [self.pick_num]
        self.choices = []
        for i in range(choices_num - 1):
            while 1:
                tmp = random.randint(0, lenm1)
                obj = data[tmp]
                if tmp not in nums_taken and obj[3] == self.pick[3]:
                    break
            nums_taken.append(tmp)
            self.choices.append((tmp, obj))
        self.choices.insert(random.randint(0, choices_num - 1), (self.pick_num, self.pick))

    def get_pick(self):
        return self.pick

    def get_choices(self):
        return [x[1] for x in self.choices]

    def answer(self, num):
        return self.choices[num][0] == self.pick_num

class DataQuizzer:
    def __init__(self, filename):
        OutWithDecoding()
        f = open(filename, 'r')
        data = '(' + f.read() + ')'
        self.data = eval(data)

    def new_question(self, choices_num = 5):
        return Question(self.data, choices_num)
